@@ -74,7 +74,7 @@ typedef struct {
 	GtkTooltips *tooltips;
 	pilot_properties properties; /* = { NULL }; */
 	state curstate;
-	GtkWidget *pixmap; 
+	GtkWidget *image; 
 	GtkWidget *dialogWindow; 
 	GtkWidget *operationDialogWindow;
 	GtkWidget *pb;
@@ -133,9 +133,10 @@ gpilotd_connect_cb (GnomePilotClient *client,
 	gchar *buf;
 	PilotApplet *applet = PILOT_APPLET (user_data);
 
-	gtk_tooltips_set_tip (applet->tooltips, GTK_WIDGET(applet) ,_("Synchronizing..."), NULL);
+	gtk_tooltips_set_tip (applet->tooltips, GTK_WIDGET(applet->applet),
+			("Synchronizing..."), NULL);
   
-	if (!GTK_WIDGET_REALIZED (applet->pixmap)) {
+	if (!GTK_WIDGET_REALIZED (applet->image)) {
 		g_warning ("! realized");
 		return;
 	}
@@ -152,12 +153,13 @@ gpilotd_connect_cb (GnomePilotClient *client,
 		applet->overall_progress_bar = glade_xml_get_widget (xml,"overall_progress_bar");
 		applet->conduit_progress_bar = glade_xml_get_widget (xml,"conduit_progress_bar");
 		applet->cancel_button        = glade_xml_get_widget (xml,"cancel_button");
+		applet->message_buffer       = gtk_text_view_get_buffer(
+				GTK_TEXT_VIEW(applet->message_area));
 
 		gtk_signal_connect (GTK_OBJECT (applet->cancel_button),"clicked",
 				   GTK_SIGNAL_FUNC (cancel_cb),applet);
 	} else {
-		gtk_text_buffer_place_cursor (applet->message_buffer,0);
-		gtk_text_buffer_set_text (applet->message_buffer, "", 0);
+		gtk_text_buffer_set_text (applet->message_buffer, "", -1);
 	}
 
 	gtk_widget_set_sensitive (applet->cancel_button, FALSE);
@@ -185,7 +187,8 @@ gpilotd_disconnect_cb (GnomePilotClient *client,
 		       gpointer user_data)
 {
 	PilotApplet *applet = PILOT_APPLET (user_data);
-	gtk_tooltips_set_tip (applet->tooltips, GTK_WIDGET(applet) ,_("Ready to synchronize"), NULL);
+	gtk_tooltips_set_tip (applet->tooltips, GTK_WIDGET(applet->applet),
+			("Ready to synchronize"), NULL);
 
 	applet->curstate = WAITING;
 	pilot_draw (applet);
@@ -195,6 +198,60 @@ gpilotd_disconnect_cb (GnomePilotClient *client,
 	}
 }
 
+/* taken from http://live.gnome.org/GnomeLove/PanelAppletTutorial */
+static void
+applet_back_change (PanelApplet			*a,
+		    PanelAppletBackgroundType	type,
+		    GdkColor			*color,
+		    GdkPixmap			*pixmap,
+		    PilotApplet			*applet) 
+{
+        GtkRcStyle *rc_style;
+        GtkStyle *style;
+
+        /* reset style */
+        gtk_widget_set_style (GTK_WIDGET (applet->applet), NULL);
+        rc_style = gtk_rc_style_new ();
+        gtk_widget_modify_style (GTK_WIDGET (applet->applet), rc_style);
+        gtk_rc_style_unref (rc_style);
+
+        switch (type) {
+                case PANEL_COLOR_BACKGROUND:
+                        gtk_widget_modify_bg (GTK_WIDGET (applet->applet),
+                                        GTK_STATE_NORMAL, color);
+                        break;
+
+                case PANEL_PIXMAP_BACKGROUND:
+                        style = gtk_style_copy (GTK_WIDGET (
+						applet->applet)->style);
+                        if (style->bg_pixmap[GTK_STATE_NORMAL])
+                                g_object_unref
+                                        (style->bg_pixmap[GTK_STATE_NORMAL]);
+                        style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref
+                                (pixmap);
+                        gtk_widget_set_style (GTK_WIDGET (applet->applet),
+                                        style);
+                        g_object_unref (style);
+                        break;
+
+                case PANEL_NO_BACKGROUND:
+                default:
+                        break;
+        }
+
+}
+
+static void gpilotd_scroll_to_insert_mark(PilotApplet *applet)
+{
+	gtk_text_view_scroll_to_mark(
+			GTK_TEXT_VIEW(applet->message_area),
+       			gtk_text_buffer_get_insert(applet->message_buffer),
+			0.2,
+			FALSE,
+			0.0,
+			0.0);
+}
+
 static void 
 gpilotd_request_completed (GnomePilotClient *client,
 			   const gchar *id,
@@ -209,6 +266,7 @@ gpilotd_request_completed (GnomePilotClient *client,
 		gtk_text_buffer_insert_at_cursor (applet->message_buffer,txt,-1);
 		g_free (txt);
 
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 
 /*	show_message_dialog ("Pilot %s has completed request %ld",
@@ -234,6 +292,7 @@ gpilotd_conduit_start (GnomePilotClient* client,
 		txt=g_strdup_printf (_("%s: Started\n"),conduit);
 		gtk_text_buffer_insert_at_cursor (applet->message_buffer,txt,-1);
 		g_free (txt);
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 }
 
@@ -254,6 +313,7 @@ gpilotd_conduit_end (GnomePilotClient* client,
 		gtk_text_buffer_insert_at_cursor (applet->message_buffer,txt,-1);
 		g_free (txt);
 		gtk_progress_set_format_string (GTK_PROGRESS (applet->conduit_progress_bar), "");
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 }
 
@@ -313,6 +373,7 @@ gpilotd_conduit_message (GnomePilotClient* client,
 		gchar *txt=g_strdup_printf ("%s: %s\n",conduit,message);
 		gtk_text_buffer_insert_at_cursor (applet->message_buffer,txt,-1);
 		g_free (txt);
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 	g_main_iteration (FALSE);
 }
@@ -329,6 +390,7 @@ gpilotd_daemon_message (GnomePilotClient* client,
 		gchar *txt=g_strdup_printf ("%s\n", message);
 		gtk_text_buffer_insert_at_cursor (applet->message_buffer,txt,-1);
 		g_free (txt);
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 	g_main_iteration (FALSE);
 }
@@ -344,6 +406,7 @@ gpilotd_daemon_error (GnomePilotClient* client,
 		gchar *txt=g_strdup_printf ("Error: %s\n", message);
 		gtk_text_buffer_insert_at_cursor (applet->message_buffer,txt,-1);
 		g_free (txt);
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 }
 
@@ -351,7 +414,7 @@ static void
 handle_client_error (PilotApplet *self)
 {
 	if (self->curstate == SYNCING) {
-		show_warning_dialog (self, _("Pilot is currently synchronizing.\nPlease wait for it to finish."));
+		show_warning_dialog (self, _("PDA is currently synchronizing.\nPlease wait for it to finish."));
 	} else {
 		self->curstate=INITIALISING;
 		gtk_tooltips_set_tip (self->tooltips, GTK_WIDGET (self->applet),_("Not connected. Please restart daemon."), NULL);	
@@ -369,12 +432,13 @@ about_cb(BonoboUIComponent *uic, PilotApplet *pilot, const gchar *verbname)
 				  "JP Rosevear <jpr@ximian.com>", 
 				  "Chris Toshok <toshok@ximian.com>",
 				  "Frank Belew <frb@ximian.com>",
+				  "Matt Davey <mcdavey@mrao.cam.ac.uk>",
 				  NULL};
 
 	about = gnome_about_new (_("gnome-pilot applet"), 
 				 VERSION,
-				 _("Copyright 2000-2003 Free Software Foundation, Inc."),
-				 _("A dnd and gui monitor for gpilotd.\n"),
+				 _("Copyright 2000-2006 Free Software Foundation, Inc."),
+				 _("A PalmOS PDA monitor.\n"),
 				 (const gchar**)authors,
 				 NULL,
 				 "Translations by the GNOME Translation Project",
@@ -457,7 +521,7 @@ help_cb(BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
 	GError *error = NULL;
 
 	gnome_help_display_on_screen  ("gnome-pilot", "pilot-applet",
-				       (GTK_WIDGET (applet->applet)),
+				       (gtk_widget_get_screen(GTK_WIDGET (applet->applet))),
 				       &error);
 
 	if (error)
@@ -470,7 +534,7 @@ help_cb(BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
 							 error->message);
 		g_error_free (error);
 		gtk_window_set_screen (GTK_WINDOW (message_dialog),
-				       gtk_widget_get_screen (GTK_WIDGET (applet)));
+				       gtk_widget_get_screen (GTK_WIDGET (applet->applet)));
 		gtk_dialog_run (GTK_DIALOG (message_dialog));
 		gtk_widget_destroy (message_dialog);
 	}
@@ -679,7 +743,7 @@ restart_cb (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
 	PilotApplet *applet = user_data;
 
 	if (applet->curstate == SYNCING) {
-		if ( gnome_dialog_run_and_close (GNOME_DIALOG (gnome_question_dialog (_("Pilot sync is currently in progress.\nAre you sure you want to restart daemon?"),NULL,NULL))) !=0 ) {
+		if ( gnome_dialog_run_and_close (GNOME_DIALOG (gnome_question_dialog (_("PDA sync is currently in progress.\nAre you sure you want to restart daemon?"),NULL,NULL))) !=0 ) {
 			return;
 		}
 		if (applet->progressDialog!=NULL) {
@@ -725,6 +789,12 @@ static void
 install_popup_menu (PilotApplet *self, gboolean on_off)
 {
 	char *menu_xml;
+	/* ensure the label strings are extracted by intltool.  This list of
+	 * strings should match the list in the menu_template below
+	 */
+#define FOO_STRING _("Restore...") _("Restart") _("Last log...") \
+           _("Preferences...") _("Help") _("About")
+#undef FOO_STRING
 	const char *menu_template = 
 		"<popup name=\"button3\">"
 		"<menuitem name=\"Restore\" verb=\"Restore\" _label=\"Restore...\""
@@ -740,7 +810,7 @@ install_popup_menu (PilotApplet *self, gboolean on_off)
 		"          pixtype=\"stock\" pixname=\"gtk-properties\"/>"
 		"<menuitem name=\"Item 2\" verb=\"Help\" _label=\"Help\""
 		"          pixtype=\"stock\" pixname=\"gtk-help\"/>"
-		"<menuitem name=\"Item 3\" verb=\"About\" _label=\"About...\""
+		"<menuitem name=\"Item 3\" verb=\"About\" _label=\"About\""
 		"          pixtype=\"stock\" pixname=\"gnome-stock-about\"/>"
 		"</popup>";
 
@@ -801,9 +871,11 @@ gpilotd_conduit_error (GnomePilotClient* client,
 							  iter,
 							  txt,-1,
 							  "foreground-gdk", 
-							  applet->errorColor);
+							  applet->errorColor,
+							  NULL);
 		g_free (txt);
 		gtk_text_iter_free (iter);
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 }
 
@@ -866,12 +938,12 @@ file_size (char *name)
 static void 
 pilot_draw (PilotApplet *self)
 {
-	if (!GTK_WIDGET_REALIZED (self->pixmap)) {
+	if (!GTK_WIDGET_REALIZED (self->image)) {
 		g_warning ("pilot_draw ! realized");
 		return;
 	}
 
-	gnome_pixmap_load_file (GNOME_PIXMAP (self->pixmap), pixmaps[self->curstate]);
+	gtk_image_set_from_file(GTK_IMAGE(self->image), pixmaps[self->curstate]);
   
 }
 
@@ -1244,7 +1316,6 @@ applet_destroy (GtkWidget *applet, PilotApplet *self)
 static void
 create_pilot_widgets (GtkWidget *widget, PilotApplet *self) 
 { 
-	GtkWidget *event_box; 
 	GtkStyle *style; 
 	int i; 
 
@@ -1267,40 +1338,39 @@ create_pilot_widgets (GtkWidget *widget, PilotApplet *self)
 	self->curstate = INITIALISING; 
 
 	for (i = 0; i < sizeof (pixmaps)/sizeof (pixmaps[0]); i++) 
-		pixmaps[i] = gnome_unconditional_pixmap_file (pixmaps[i]); 
+		pixmaps[i] = gnome_program_locate_file(
+		    NULL, GNOME_FILE_DOMAIN_PIXMAP, pixmaps[i], TRUE, NULL);
 
-	event_box = gtk_event_box_new (); 
+	self->image = gtk_image_new_from_file (pixmaps[self->curstate]); 
 
-	self->pixmap = gnome_pixmap_new_from_file (pixmaps[self->curstate]); 
-
-	gtk_widget_set_events (event_box, gtk_widget_get_events (widget) | 
-			       GDK_BUTTON_PRESS_MASK); 
-	gtk_signal_connect (GTK_OBJECT (event_box), "button-press-event", 
+	gtk_signal_connect (GTK_OBJECT (widget), "button-press-event", 
 			    GTK_SIGNAL_FUNC (pilot_clicked_cb), self); 
 
 
-	gtk_widget_show (self->pixmap);   
-	gtk_container_add (GTK_CONTAINER (event_box), self->pixmap); 
+	gtk_widget_show (self->image);   
 
-	gtk_container_add (GTK_CONTAINER (widget), event_box);
+	gtk_container_add (GTK_CONTAINER (widget), self->image);
 
 	self->timeout_handler_id = gtk_timeout_add (1000,(GtkFunction)timeout,self); 
 
 	gtk_signal_connect (GTK_OBJECT (widget),"destroy", 
 			    GTK_SIGNAL_FUNC (applet_destroy),self); 
 
+	gtk_signal_connect (GTK_OBJECT (widget),"change_background", 
+			    GTK_SIGNAL_FUNC (applet_back_change),self); 
+
 	/* FIXME */
 	/* XXX change_orient */
 	/* XXX change_size */
 
 	/* Now set the dnd features */
-	gtk_drag_dest_set (GTK_WIDGET (self->pixmap), 
+	gtk_drag_dest_set (GTK_WIDGET (self->image), 
 			   GTK_DEST_DEFAULT_ALL, 
 			   drop_types, n_drop_types, 
 			   GDK_ACTION_COPY); 
 
 
-	gtk_signal_connect (GTK_OBJECT (self->pixmap), 
+	gtk_signal_connect (GTK_OBJECT (self->image), 
 			    "drag_data_received", 
 			    GTK_SIGNAL_FUNC (dnd_drop_internal), 
 			    self); 
