@@ -183,6 +183,7 @@ init (GnomePilotCapplet *gpcap)
 	init_widgets (gpcap);
 
  error:
+	;
 }
 
 GnomePilotCapplet *
@@ -207,7 +208,8 @@ get_widgets (GnomePilotCapplet *gpcap)
 
 #define GW(name) glade_xml_get_widget (priv->xml, name)
 
-	gtk_window_set_title (GTK_WINDOW (gpcap), _("Pilot Settings"));
+	gtk_window_set_title (GTK_WINDOW (gpcap), _("gnome-pilot Settings"));
+	gtk_dialog_set_has_separator (GTK_DIALOG (gpcap), FALSE);
 	gtk_dialog_add_button (GTK_DIALOG (gpcap),
 			       GTK_STOCK_HELP, GTK_RESPONSE_HELP);
 	gtk_dialog_add_button (GTK_DIALOG (gpcap),
@@ -378,7 +380,7 @@ gnome_pilot_capplet_create_pilots_treeview (char *name, char *string1, char *str
 						     renderer, "text", 0, NULL);
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("ID"),
 						     renderer, "text", 1, NULL);
-	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("UserName"),
+	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("Owner"),
 						     renderer, "text", 2, NULL);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
@@ -410,10 +412,8 @@ append_devices_treeview (GnomePilotCapplet *gpcap, GPilotDevice *device, GtkTree
 	gtk_list_store_append (GTK_LIST_STORE (priv->devices_model), iter);
 	gtk_list_store_set (GTK_LIST_STORE (priv->devices_model), iter,
 			    0, device->name,
-			    1, device->port,
-			    2, device->speed,
-			    3, device_type_to_str (device->type),
-			    4, device,
+			    1, device_type_to_str (device->type),
+			    2, device,
 			    -1);
 }
 
@@ -452,18 +452,14 @@ gnome_pilot_capplet_create_devices_treeview (char *name, char *string1, char *st
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled),
 					     GTK_SHADOW_IN);
 
-	model = gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_POINTER);
+	model = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
 	treeview = gtk_tree_view_new_with_model ((GtkTreeModel *) model);
 
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("Name"),
 						     renderer, "text", 0, NULL);
-	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("Port"),
-						     renderer, "text", 1, NULL);
-	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("Speed"),
-						     renderer, "text", 2, NULL);
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, _("Type"),
-						     renderer, "text", 3, NULL);
+						     renderer, "text", 1, NULL);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
@@ -771,7 +767,7 @@ gpcap_pilots_delete (GtkWidget *widget, gpointer user_data)
 
 	dlg = gtk_message_dialog_new (GTK_WINDOW (gpcap), GTK_DIALOG_DESTROY_WITH_PARENT, 
 				      GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
-				      _("Are you sure you want to delete %s pilot?"), pilot->name);
+				      _("Are you sure you want to delete PDA `%s'?"), pilot->name);
 
 	if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_YES) {
 		gtk_list_store_remove (GTK_LIST_STORE (priv->pilots_model),
@@ -795,11 +791,11 @@ gpcap_pilots_popup (GtkTreeView *treeview, GdkEventButton *event, gpointer user_
 	GnomePilotCappletPrivate *priv;
 
 	GnomeUIInfo popup [] = {
-		GNOMEUIINFO_ITEM_DATA (N_("_Edit this pilot..."),
-				       N_("Edit the currently selected pilot"),
+		GNOMEUIINFO_ITEM_DATA (N_("_Edit this PDA..."),
+				       N_("Edit the currently selected PDA"),
 				       gpcap_pilots_edit, gpcap, NULL),
-		GNOMEUIINFO_ITEM_DATA (N_("_Delete this pilot"),
-				       N_("Delete the currently selected pilot"),
+		GNOMEUIINFO_ITEM_DATA (N_("_Delete this PDA"),
+				       N_("Delete the currently selected PDA"),
 				       gpcap_pilots_delete, gpcap, NULL),
 		GNOMEUIINFO_END
 	};
@@ -839,7 +835,7 @@ get_current_device (GnomePilotCapplet *gpcap, GtkTreeIter *iter)
 		return NULL;
 
 	gtk_tree_model_get (priv->devices_model, iter,
-			    4, &device,
+			    2, &device,
 			    -1);
 
 	return device;
@@ -910,9 +906,7 @@ gpcap_devices_edit (GtkWidget *widget, gpointer user_data)
 	if (res) {
 		gtk_list_store_set (GTK_LIST_STORE (priv->devices_model), &iter,
 				    0, device->name,
-				    1, device->port,
-				    2, device->speed,
-				    3, device_type_to_str (device->type),
+				    1, device_type_to_str (device->type),
 				    -1);
 		
 		gpcap_save_state (gpcap);
