@@ -34,6 +34,9 @@
 #endif /* _XOPEN_SOURCE  */
 #endif
 
+#define _BSD_SOURCE 1		/* Or gethostname won't be declared properly
+				   on Linux and GNU platforms. */
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -47,10 +50,18 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <fcntl.h>
+#include <regex.h>
 
 #include <libxml/xmlmemory.h>
 #include <libxml/entities.h>
 
+#ifdef WITH_HAL
+#include <dbus/dbus.h>
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
+#include <libhal.h>
+#endif
+
 #include <libgnome/libgnome.h>
 
 #include <pi-source.h>
@@ -65,43 +76,50 @@
 
 #include <libgpilotdCM/gnome-pilot-conduit-management.h>
 
+static GPilotContext *context = NULL;
+
+#ifdef WITH_HAL
+static DBusConnection *dbus_connection = NULL;
+static LibHalContext *hal_ctx = NULL;
+
+static gboolean gpilotd_dbus_init (void);
+static LibHalContext *gpilotd_hal_init (void);
+#endif
+static guint visor_timeout_id = -1;
+static guint hal_initialised = 0;
+
+
 /* Set to true when the config should be reloaded */
-gboolean reread_config;
-guint visor_timeout_id = -1;
+static gboolean reread_config = FALSE;
 
-gint device_equal_by_io (GPilotDevice *,GIOChannel *);
-gboolean device_in (GIOChannel *,
-		   GIOCondition ,
-		   GPilotContext *);
-gboolean device_err (GIOChannel *,
-		    GIOCondition ,
-		    GPilotContext *);
-gboolean network_device_in (GIOChannel *,
-		   GIOCondition ,
-		   GPilotContext *);
-gboolean network_device_err (GIOChannel *,
-		    GIOCondition ,
-		    GPilotContext *);
-void monitor_channel (GPilotDevice *,GPilotContext *);
-void remove_pid_file (void);
+static gboolean device_in (GIOChannel *, GIOCondition, GPilotContext *);
+static gboolean device_err (GIOChannel *, GIOCondition, GPilotContext *);
+static gboolean network_device_in (GIOChannel *, GIOCondition, GPilotContext *);
+static gboolean network_device_err (GIOChannel *, GIOCondition, GPilotContext *);
+static void monitor_channel (GPilotDevice *, GPilotContext *);
+static void remove_pid_file (void);
+#ifdef PILOT_LINK_0_12
+static int check_usb_config(GPilotDevice *device);
+#endif
 
-gint 
-device_equal_by_io (GPilotDevice *dev,GIOChannel *io) 
+static int
+device_equal_by_io (GPilotDevice *dev, GIOChannel *io) 
 {
-	return !(dev->io==io);
+	return !(dev->io == io);
 }
 
 static void
-remove_device (GPilotContext *context,GPilotDevice *device) 
+remove_device (GPilotContext *context, GPilotDevice *device) 
 {
 	GList *l;
 
-	g_message ("Removing %s",device->name);
+	g_message ("Removing %s", device->name);
 
-	l = g_list_find (context->devices,device);
+	l = g_list_find (context->devices, device);
 	if (l != NULL) {
 		gpilot_device_free (l->data);
-		g_list_remove (context->devices,l);
+		context->devices = g_list_remove (context->devices, l);
+		g_list_free_1 (l);
 	} else {
 		g_message ("%s not found",device->name);
 	}
@@ -110,9 +128,10 @@ remove_device (GPilotContext *context,GPilotDevice *device)
 static void 
 pilot_set_baud_rate (GPilotDevice *device) 
 {
-	static gchar rate_buf[128];
-	g_snprintf (rate_buf,128,"PILOTRATE=%d", device->speed);
-	g_message ("setting %s",rate_buf);
+	static char rate_buf[128];
+	
+	g_snprintf (rate_buf, 128, "PILOTRATE=%d", device->speed);
+	g_message ("setting %s", rate_buf);
 	putenv (rate_buf);
 }
 
@@ -122,14 +141,25 @@ pilot_set_baud_rate (GPilotDevice *device)
 static int 
 pilot_connect (GPilotDevice *device,int *error) 
 {
+#define MAX_TIME_FOR_PI_BIND 1000000 /* 1 second, or 1,000,000 usec. */
+#define SLEEP_TIME_FOR_PI_BIND 50000 /* 50 ms */
+	
+#ifndef PILOT_LINK_0_12
 	struct pi_sockaddr addr;
+	union {
+		struct pi_sockaddr *pi_sockaddrp;
+		struct sockaddr *sockaddrp;
+	} addrp;
+#endif
 	int sd, listen_sd, pf;
 	int ret;
-    
+
+	int time_elapsed_pi_bind = 0;
+	
 	if (device->type != PILOT_DEVICE_NETWORK) {
 		pilot_set_baud_rate (device);
 	}
-
+	
 	switch (device->type) {
 	case PILOT_DEVICE_SERIAL:
 		pf = PI_PF_PADP;
@@ -148,85 +178,133 @@ pilot_connect (GPilotDevice *device,int *error)
 		break;
 	}
 	
-	if (!(listen_sd = pi_socket (PI_AF_PILOT, PI_SOCK_STREAM, pf))) {
-		g_warning ("pi_socket: %s",strerror (errno));
-		if (error) *error = 1;
-		return -1;
-	}
-	
-	addr.pi_family = PI_AF_PILOT;
-
-/*
-	Most important for networking
-	. resolves inside libpisock as network
-	It is done earlier in gpilotd_device_init
-	so don't really need to do it again here.
-*/
 	if (device->type == PILOT_DEVICE_NETWORK) {
-		device->port = "net:any";
+		/* In the case of network pi_sockets, the socket is already
+		 * listening at this point, so move on to accept */
+		listen_sd = device->fd;
+	} else {
+#ifdef PILOT_LINK_0_12
+		/* pl 0.12.0 wants to autodetect the protocol, so pass DLP */
+		/* (at time of writing there's a buglet in pl where if you
+		 * _do_ pass in the correct NET protocol then pl will flush
+		 * pending net input, which might just lose your first
+		 * packet.
+		 */
+		listen_sd = pi_socket (PI_AF_PILOT, PI_SOCK_STREAM, PI_PF_DLP);
+#else
+		listen_sd = pi_socket (PI_AF_PILOT, PI_SOCK_STREAM, pf);
+#endif
+		if (listen_sd < 0) {
+			g_warning ("pi_socket returned error %d (%s)",
+			    listen_sd,
+			    strerror (errno));
+			if (error) *error = 1;
+			return -1;
+		}
+
+		
+		/* 
+		 * When using HAL and USB, the ttyUSB[01] device files are
+		 * not created until after Hal has notified gpilotd that
+		 * a file exists. This causes pi_bind to fail.
+		 * To prevent a failure (and therefore sync), retry pi_bind
+		 * for up to the time allotted, sleeping a little in-between
+		 * tries.
+		 */
+		
+		time_elapsed_pi_bind = 0;
+		do {
+#ifdef PILOT_LINK_0_12
+			ret = pi_bind (listen_sd, device->port);
+#else
+			addr.pi_family = PI_AF_PILOT;
+			strcpy (addr.pi_device,device->port);
+			addrp.pi_sockaddrp = &addr;
+			ret = pi_bind (listen_sd, addrp.sockaddrp,
+			    sizeof (addr));	
+#endif
+			if(ret < 0) {
+				usleep(SLEEP_TIME_FOR_PI_BIND);
+				time_elapsed_pi_bind += SLEEP_TIME_FOR_PI_BIND;
+			}
+		} while(ret<0 && time_elapsed_pi_bind < MAX_TIME_FOR_PI_BIND);
+		
+		if (ret < 0) {
+			g_warning (_("Unable to bind to PDA"));
+			if (error)
+				*error = 1;
+			pi_close(listen_sd);
+#ifdef PILOT_LINK_0_12
+			check_usb_config(device);
+#endif
+			return 0;
+		}
+
+		ret = pi_listen (listen_sd, 1);
+		if (ret != 0) {
+			g_warning ("pi_listen: %s", strerror (errno));
+			if (error)
+				*error = 2;
+			pi_close(listen_sd);
+			return 0;
+		}
 	}
 
-	strcpy (addr.pi_device,device->port);
-
-	ret = pi_bind (listen_sd, (struct sockaddr*)&addr, sizeof (addr));
-	if (ret == -1) {
-		g_warning (_("Unable to bind to pilot"));
-		if (error)
-			*error = 1;
-		pi_close(listen_sd);
-		return 0;
-	}
-
-	ret = pi_listen (listen_sd, 1);
-	if (ret != 0) {
-		g_warning ("pi_listen: %s", strerror (errno));
-		if (error)
-			*error = 2;
-		pi_close(listen_sd);
-		return 0;
-	}
-
-	sd = pi_accept_to (listen_sd, NULL,0, device->timeout); 
-	if (sd == -1) {
-		g_warning ("pi_accept_to: %s", strerror (errno));
+	sd = pi_accept_to (listen_sd, NULL, NULL, device->timeout); 
+	if (sd < 0) {
+		g_warning ("pi_accept_to returned %d: %s", sd, strerror (errno));
 		g_warning ("pi_accept_to: timeout was %d secs", device->timeout);
 		if (error)
 			*error = 2;
-		pi_close(listen_sd);
+		if (device->type != PILOT_DEVICE_NETWORK) {
+			pi_close(listen_sd);
+		}
 		return 0;
 	}
 
 	if (error)
 		*error = 0;
-	pi_close(listen_sd);
-	
+
+	/* since pilot-link 0.12.0preX, the listen_sd represents
+	 * the accepted connection, so don't close listen_sd
+	 * after the accept */
+#ifndef PILOT_LINK_0_12
+	if (device->type != PILOT_DEVICE_NETWORK) {
+		pi_close(listen_sd);
+	}		
+#endif
 	return sd;
 }
 
-static void pilot_disconnect (int sd)
+static void
+pilot_disconnect (int sd)
 {
+	int ret; 
+
 	dlp_EndOfSync (sd, 0);
-	pi_close (sd);
+	ret = pi_close (sd);
+	if (ret < 0) {
+		g_warning ("error %d from pi_close.", ret);
+	}
+
 }
 
-static void write_sync_stamp (GPilotPilot *pilot,
-			     int pfd,
-			     struct PilotUser *pu,
-			     guint32 last_sync_pc, 
-			     time_t t)
+static void
+write_sync_stamp (GPilotPilot *pilot, int pfd, struct PilotUser *pu,
+		  guint32 last_sync_pc, time_t t)
 {
-	gchar prefix[256];
-
-	pu->lastSyncPC=last_sync_pc;
-	pu->lastSyncDate=t;
-
-	g_snprintf (prefix,255,"/gnome-pilot.d/gpilotd/Pilot%d/",pilot->number);
+	char prefix[256];
+	
+	pu->lastSyncPC = last_sync_pc;
+	pu->lastSyncDate = t;
+	
+	g_snprintf (prefix, 255, "/gnome-pilot.d/gpilotd/Pilot%d/", pilot->number);
 	gnome_config_push_prefix (prefix);
-	gnome_config_private_set_int ("sync_date",t);
+	gnome_config_private_set_int ("sync_date", t);
 	gnome_config_pop_prefix ();
 	gnome_config_sync ();
-
-	dlp_WriteUserInfo (pfd,pu);
+	
+	dlp_WriteUserInfo (pfd, pu);
 }
 
 /** pilot lookup methods **/
@@ -255,6 +333,74 @@ match_pilot_and_majick (const GPilotPilot *pilot,
 	return 0;
 }
 
+
+#ifdef PILOT_LINK_0_12
+/* Checks whether visor module is loaded but 'usb:' device is selected,
+ * or vice versa.  Returns 0 if the config looks okay, or -1 otherwise.
+ * raises a gpilot_gui_warning_dialog and a g_warning if the config
+ * looks incorrect.
+ */
+static int
+check_usb_config(GPilotDevice *device) {
+#define MAXLINELEN 256
+	char line[MAXLINELEN];
+	gboolean visor_loaded;
+	gboolean libusb_device;
+	FILE *f;
+	int error = 0;
+
+	if (device->type != PILOT_DEVICE_USB_VISOR)
+		return 0;
+	visor_loaded = FALSE;
+	libusb_device = TRUE; /* defaults, in case we skip block */
+	f = fopen ("/proc/modules", "r");
+	if (f) {
+		while(fgets(line, MAXLINELEN, f) != NULL) {
+			if (strncmp(line, "visor", 5) == 0) {
+				visor_loaded = TRUE;
+				break;
+			}
+		}
+		fclose(f);
+		libusb_device = (strncmp(device->port,
+					 "usb:", 4) == 0);
+	}
+	if (libusb_device && visor_loaded) {
+		g_snprintf(line, MAXLINELEN,_("Failed to connect using "
+					      "device `%s', on port `%s'.  "
+					      "Check your configuration, "
+					      "as you requested "
+					      "new-style libusb `usb:' "
+					      "syncing, but have the "
+					      "old-style `visor' kernel "
+					      "module loaded.  "
+					      "You may need to select a "
+					      "`ttyUSB...' device."),
+			   device->name, device->port);
+		g_warning(line);
+		gpilot_gui_warning_dialog(line);
+		error = 1;
+	} else if (!libusb_device && !visor_loaded) {
+		g_snprintf(line, MAXLINELEN,_("Failed to connect using "
+					      "device `%s', on port `%s'.  "
+					      "Check your configuration, "
+					      "as you requested "
+					      "old-style usbserial `ttyUSB' "
+					      "syncing, but do not have the "
+					      "usbserial `visor' kernel "
+					      "module loaded.  "
+					      "You may need to select a "
+					      "`usb:' device."),
+			   device->name, device->port);
+		g_warning(line);
+		gpilot_gui_warning_dialog(line);
+		error = 1;
+	}
+	return error;
+}
+#endif /* PILOT_LINK_0_12 */
+
+
 /**************************/
 
 /*
@@ -280,9 +426,9 @@ gpilot_attempt_restore (struct PilotUser pu,
 	dlp_ReadStorageInfo (pfd, 0, &cardinfo);
 	dlp_ReadSysInfo (pfd, &sysinfo);
 
-	if (g_list_length (context->pilots) == 1) {
+	if (context->pilots && context->pilots->next == NULL) {
 		pilot = GPILOT_PILOT (g_list_nth (context->pilots, 0)->data);
-		g_message ("D: Only one pilot (%s) profile...", pilot->name);
+		g_message ("D: Only one PDA (%s) profile...", pilot->name);
 	} else {
 		for (iterator = context->pilots; iterator; iterator = g_list_next (iterator)) {
 			pilot = GPILOT_PILOT (iterator->data);
@@ -305,7 +451,7 @@ gpilot_attempt_restore (struct PilotUser pu,
 	} else {
 		/* MUST GO */
 		gpilot_gui_warning_dialog ("no ident\n"
-					   "restoring pilot with ident\n"
+					   "restoring PDA with ident\n"
 					   "c/r = %lu/%lu, exciting things\n"
 					   "will soon be here...",
 					   cardinfo.creation,
@@ -319,27 +465,27 @@ gpilot_attempt_restore (struct PilotUser pu,
  * This function handles when sync_device (...) encounters an unknown pilot
  */
 
- static void
+static void
 gpilot_syncing_unknown_pilot (struct PilotUser pu, 
 			      int pfd,
 			      GPilotDevice *device, 
 			      GPilotContext *context)
 {
 
-	g_warning (_("Unknown pilot, no userID/username match %ld"),pu.userID);
+	g_warning (_("Unknown PDA, no userID/username match %ld"),pu.userID);
 	/* FIXME: here, restoring one of the available pilots should be
 	   offered to the user. Of course with password prompt if the user
 	   has password set
 	   bug # 8217 */
 	if (pu.userID == 0) {
 		if (gpilot_attempt_restore (pu, pfd, device, context) == FALSE) {
-			gpilot_gui_warning_dialog (_("Use gnomecc to configure pilot"));
+			gpilot_gui_warning_dialog (_("Use gnomecc to configure PDA"));
 		}
 	} else {
 		/* FIXME: here we should offer to create a profile for the pilot,
 		   bug # 8218 */
-		gpilot_gui_warning_dialog (_("Unknown pilot - no pilots matches ID %ld\n"
-					     "Use gnomecc to set pilot's ID"),pu.userID);
+		gpilot_gui_warning_dialog (_("Unknown PDA - no PDA matches ID %ld\n"
+					     "Use gnomecc to configure gnome-pilot"),pu.userID);
 	}
 }
 
@@ -355,18 +501,19 @@ do_cradle_events (int pfd,
 		 struct PilotUser *pu,
 		 GPilotDevice *device) 
 {
-	GList *events,*it;
+	GList *events, *it;
 	gboolean ret = TRUE;
-
+	
 	/* elements in events freed by gpc_request_purge calls
 	   in orbed_notify_completion */
 	events = gpc_queue_load_requests_for_cradle (device->name);
-
-	g_message (_("Cradle %s has %d events"), device->name, g_list_length (events));
-
+	
+	g_message (_("Device %s has %d events"), device->name, g_list_length (events));
+	
 	/* if no events, return FALSE */
-	if (g_list_length (events)==0) ret = FALSE;
-
+	if (!events)
+		return FALSE;
+	
 	it = events;
 	
 	while (it) {
@@ -448,7 +595,7 @@ do_sync (int pfd,
 {
 	GList *conduit_list, *backup_conduit_list, *file_conduit_list;
 	GnomePilotSyncStamp stamp;
-	gchar *pilot_name;
+	char *pilot_name;
 
 	pilot_name = pilot_name_from_id (pu->userID,context);
 
@@ -460,18 +607,18 @@ do_sync (int pfd,
 	stamp.sync_PC_Id=context->sync_PC_Id;
 
 	if (device->type == PILOT_DEVICE_NETWORK) {
-		g_message (_("NetSync request detected, synchronizing pilot"));
+		g_message (_("NetSync request detected, synchronizing PDA"));
 	} else {
-		g_message (_("HotSync button pressed, synchronizing pilot"));
+		g_message (_("HotSync button pressed, synchronizing PDA"));
 	}
-	g_message (_("Pilot ID is %ld, name is %s, owner is %s"),
+	g_message (_("PDA ID is %ld, name is %s, owner is %s"),
 		  pu->userID,
 		  pilot->name,
 		  pu->username);
   
 	/* Set a log entry in the pilot */
 	{
-		gchar hostname[64];
+		char hostname[64];
 		gpilot_add_log_entry (pfd,"gnome-pilot v.%s\n",VERSION);
 		if (gethostname (hostname,63)==0)
 			gpilot_add_log_entry (pfd,_("On host %s\n"),hostname);
@@ -539,18 +686,18 @@ gpilot_syncing_known_pilot (GPilotPilot *pilot,
 		   on the pilot */
 		if (pilot->passwd) {
 			char *pwd;
-			pwd = (char*)g_malloc (pu.passwordLength);
-			strncpy (pwd,pu.password,pu.passwordLength);
-			if (g_strcasecmp (pilot->passwd,(char*)crypt (pwd,pilot->passwd))) {
+			
+			pwd = g_strndup (pu.password, pu.passwordLength);
+			if (g_strcasecmp (pilot->passwd, crypt (pwd, pilot->passwd))) {
 				pwd_ok = FALSE;
-				gpilot_gui_warning_dialog (_("Unknown pilot - no pilots matches ID %ld\n"
-							     "Use gnomecc to set pilot's ID"),pu.userID);
+				gpilot_gui_warning_dialog (_("Unknown PDA - no PDA matches ID %ld\n"
+							     "Use gpilotd-control-applet to set PDA's ID"), pu.userID);
 			}
 			g_free (pwd);
-		} 
+		}
 		
 		if (pwd_ok) {
-			do_sync (pfd,context,&pu,pilot,device);
+			do_sync (pfd, context, &pu, pilot, device);
 		}
 	}
 }
@@ -579,12 +726,11 @@ static gboolean
 sync_device (GPilotDevice *device, GPilotContext *context)
 {
 	GPilotPilot *pilot;
-	
-	int pfd;
 	int connect_error;
 	struct PilotUser pu;
 	struct SysInfo ps;
-
+	int pfd;
+	
 	g_assert (context != NULL);
 	g_return_val_if_fail (device != NULL, FALSE);
 
@@ -595,17 +741,21 @@ sync_device (GPilotDevice *device, GPilotContext *context)
                /* connect succeeded, try to read the systeminfo */
                if (dlp_ReadSysInfo (pfd, &ps) < 0) {
                        /* no ? drop connection then */
-                       g_warning (_("An error occured while getting the pilot's system data"));
+                       g_warning (_("An error occurred while getting the PDA's system data"));
+#ifdef PILOT_LINK_0_12
+		       check_usb_config(device);
+#endif
+
 
 		/* connect succeeded, try to read the userinfo */
 		} else if (dlp_ReadUserInfo (pfd,&pu) < 0) {
 			/* no ? drop connection then */
-			g_warning (_("An error occured while getting the pilot's user data"));
+			g_warning (_("An error occurred while getting the PDA's user data"));
 		} else {
 	
 			/* If there are cradle specific events, handle them and stop */
 			if (do_cradle_events (pfd,context,&pu,device)) {
-				g_message (_("Completed events for cradle %s (%s)"),device->name,device->port);
+				g_message (_("Completed events for device %s (%s)"),device->name,device->port);
 			} else {
 				/* No cradle events, validate pilot */
 				pilot = gpilot_find_pilot_by_id (pu.userID,context->pilots);
@@ -622,29 +772,40 @@ sync_device (GPilotDevice *device, GPilotContext *context)
 			}
 		}
 		pilot_disconnect (pfd);
+		/* now restart the listener.  fairly brute force
+		 * approach, but ensures we re-initialise the listening
+		 * socket correctly.  */
+		if (device->type == PILOT_DEVICE_NETWORK) {
+			reread_config = TRUE;
+		}
 	} else {
 		if (connect_error==1) return FALSE; /* remove this device */
-		else return TRUE;
+		else {
+			if (device->type == PILOT_DEVICE_NETWORK) {
+				/* fix broken pisock sockets */
+				reread_config = TRUE;
+			}
+			return TRUE;
+		}
 	}
 
 	return TRUE;
 }
 
-gboolean 
-device_in (GIOChannel *io_channel,
-	  GIOCondition condition,
-	  GPilotContext *context) {
+static gboolean 
+device_in (GIOChannel *io_channel, GIOCondition condition, GPilotContext *context)
+{
 	GPilotDevice *device;
 	GList *element;
 	gboolean result = TRUE;
-
+	
 	g_assert (context != NULL);
-      
+	
 	element = g_list_find_custom (context->devices,
 				     io_channel,
 				     (GCompareFunc)device_equal_by_io);
-
-	if (element==NULL || element->data == NULL) {
+	
+	if (element == NULL || element->data == NULL) {
 		g_warning ("cannot find device for active IO channel");
 		return FALSE;
 	}
@@ -653,9 +814,9 @@ device_in (GIOChannel *io_channel,
 	if (context->paused) {
 		return FALSE; 
 	}	
-	g_message (_("Woke on %s"),device->name);
-	result = sync_device (device,context);
-
+	g_message (_("Woke on %s"), device->name);
+	result = sync_device (device, context);
+	
 #ifdef WITH_IRDA
 	if (device->type == PILOT_DEVICE_IRDA) {
 		g_message ("Restarting irda funk...");
@@ -665,20 +826,19 @@ device_in (GIOChannel *io_channel,
 		result = FALSE;
 	}
 #endif /* WITH_IRDA */
-
+	
 	return result;
 }
 
-gboolean 
-device_err (GIOChannel *io_channel,
-	   GIOCondition condition,
-	   GPilotContext *context) {
+static gboolean 
+device_err (GIOChannel *io_channel, GIOCondition condition, GPilotContext *context)
+{
 	GPilotDevice *device;
 	GList *element;
 	char *tmp;
-
+	
 	g_assert (context != NULL);
-
+	
 	switch (condition) {
 	case G_IO_IN: tmp = g_strdup_printf ("G_IO_IN"); break;
 	case G_IO_OUT : tmp = g_strdup_printf ("G_IO_OUT"); break;
@@ -690,8 +850,8 @@ device_err (GIOChannel *io_channel,
 	}
 	
 	element = g_list_find_custom (context->devices,io_channel,(GCompareFunc)device_equal_by_io);
-
-	if (element==NULL) {
+	
+	if (element == NULL) {
 		/* We most likely end here if the device has just been removed.
 		   Eg. start gpilotd with a monitor on a XCopilot fake serial port,
 		   kill xcopilot and watch things blow up as the device fails */
@@ -701,23 +861,21 @@ device_err (GIOChannel *io_channel,
 	}
 	
 	device = element->data;
-
+	
 	gpilot_gui_warning_dialog ("Device error on %s (%s)\n"
-				  "Caught %s",device->name,device->port,tmp); 
-	g_warning ("Device error on %s (%s), caught %s",device->name,device->port,tmp);
-
-	remove_device (context,device);
+				  "Caught %s", device->name, device->port, tmp); 
+	g_warning ("Device error on %s (%s), caught %s", device->name, device->port, tmp);
+	
+	remove_device (context, device);
 	g_free (tmp);
-		
+	
 	return FALSE;
 }
 
 #ifdef WITH_NETWORK
-gboolean 
-network_device_in (GIOChannel *io_channel,
-	  GIOCondition condition,
-	  GPilotContext *context) {
-
+static gboolean 
+network_device_in (GIOChannel *io_channel, GIOCondition condition, GPilotContext *context)
+{
 	GPilotDevice *device;
 	GList *element;
 	gboolean result = TRUE;
@@ -737,23 +895,21 @@ network_device_in (GIOChannel *io_channel,
 	if (context->paused) {
 		return FALSE; 
 	}	
-	g_message (_("Woke on %s"),device->name);
+	g_message (_("Woke on network: %s"),device->name);
 	result = sync_device (device,context);
 
 	return result;
 }
 
-gboolean 
-network_device_err (GIOChannel *io_channel,
-	   GIOCondition condition,
-	   GPilotContext *context) {
-
+static gboolean 
+network_device_err (GIOChannel *io_channel, GIOCondition condition, GPilotContext *context)
+{
 	GPilotDevice *device;
 	GList *element;
 	char *tmp;
-
+	
 	g_assert (context != NULL);
-
+	
 	switch (condition) {
 		case G_IO_IN: tmp = g_strdup_printf ("G_IO_IN"); break;
 		case G_IO_OUT : tmp = g_strdup_printf ("G_IO_OUT"); break;
@@ -765,8 +921,8 @@ network_device_err (GIOChannel *io_channel,
 	}
 	
 	element = g_list_find_custom (context->devices,io_channel,(GCompareFunc)device_equal_by_io);
-
-	if (element==NULL) {
+	
+	if (element == NULL) {
 		/* We most likely end here if the device has just been removed.
 		   Eg. start gpilotd with a monitor on a XCopilot fake serial port,
 		   kill xcopilot and watch things blow up as the device fails */
@@ -777,29 +933,40 @@ network_device_err (GIOChannel *io_channel,
 	
 	device = element->data;
 
-	gpilot_gui_warning_dialog ("Device error on %s (%s)\n"
-				  "Caught %s",device->name,device->port,tmp); 
-	g_warning ("Device error on %s (%s), caught %s",device->name,device->port,tmp);
+	gpilot_gui_warning_dialog ("Device error on %s, caught %s",
+	    device->name, tmp);
+	g_warning ("Device error on %s, caught %s",device->name, tmp);
 
-	remove_device (context,device);
+	remove_device (context, device);
 	g_free (tmp);
-		
+	
 	return FALSE;
 }
 #endif /* WITH_NETWORK */
 
-#ifdef WITH_USB_VISOR
-#ifdef linux
 static GPtrArray *vendor_product_ids = NULL;
 static GArray *product_net = NULL;
 
+static int
+known_usb_device(gchar *match_str)
+{
+	int i;
+	for (i = 0; i < vendor_product_ids->len; i++) {
+		if (!g_strncasecmp (match_str, 
+			vendor_product_ids->pdata[i], 
+			strlen (vendor_product_ids->pdata[i])))
+			return i;
+	}
+	return -1;
+}
+
 static void
 load_devices_xml (void) 
 {
 	xmlDoc *doc = NULL;
 	xmlNode *root, *node;
 	char *filename;
-	
+
 	if (vendor_product_ids)
 		return;
 	
@@ -861,18 +1028,234 @@ load_devices_xml (void)
 	xmlFreeDoc (doc);
 }
 
+#ifdef WITH_HAL
+static void
+hal_device_added (LibHalContext *ctx, const char *udi)
+{
+	gboolean visor_net = FALSE;
+	char *bus, *match_str;
+	int vendor_id, product_id, i;
+	GPilotDevice *device;
+	DBusError error;
+	GList *dev;
+
+	if (context->paused) 
+		return;
+
+	load_devices_xml ();
+
+	/* HAL match rule: we look for info.bus == 'usb_device'
+	 * and then try to match the usb_device.product_id and usb_device.vendor_id
+	 * against the list in devices.xml.
+	 */
+	if (!(bus = libhal_device_get_property_string (hal_ctx, udi, "info.bus", NULL)))
+		return;
+	if (strcmp (bus, "usb_device") != 0) {
+		libhal_free_string (bus);
+		return;
+	}
+
+	dbus_error_init (&error);
+	vendor_id = libhal_device_get_property_int (hal_ctx, udi,
+	    "usb_device.vendor_id", &error);
+	if(vendor_id == -1) {
+		g_warning ("Could not get usb vendor ID from hal: %s", error.message);
+		return;
+	}
+	product_id = libhal_device_get_property_int (hal_ctx, udi,
+	    "usb_device.product_id", NULL);
+	if(product_id == -1) {
+		g_warning ("Could not get usb product ID from hal: %s", error.message);
+		return;
+	}
+	
+	/* now look for a vendor/product match */
+	match_str = g_strdup_printf ("Vendor=%04x ProdID=%04x",
+	    vendor_id, product_id);
+	i = known_usb_device(match_str);
+	g_free(match_str);
+	if(i == -1)
+		return;
+
+	visor_net = g_array_index (product_net, gboolean, i);
+	dev = context->devices;
+	while (dev != NULL) {
+		device = dev->data;
+		if (device->type == PILOT_DEVICE_USB_VISOR) {
+			if (!visor_net)
+				device->type = PILOT_DEVICE_SERIAL;
+			
+			/* just try to sync.  Until I can talk to 
+			 * the kernel guys this is the best way to 
+			 * go. */
+			sync_device (device, context);
+
+			if (!visor_net)
+				device->type = PILOT_DEVICE_USB_VISOR;
+			
+			break;
+		}
+		
+		dev = dev->next;
+	}
+}
+
+static void
+hal_device_removed (LibHalContext *ctx, const char *udi)
+{
+	
+}
+
+static gboolean
+reinit_dbus (gpointer user_data)
+{
+	if (gpilotd_dbus_init ()) {
+		if ((hal_ctx = gpilotd_hal_init ()))
+			libhal_ctx_set_dbus_connection (hal_ctx, dbus_connection);
+		else
+			exit (1);
+		
+		return FALSE;
+	}
+	
+	return TRUE;
+}
+
+static DBusHandlerResult
+gpilotd_dbus_filter_function (DBusConnection *connection, DBusMessage *message, void *user_data)
+{
+	if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected") &&
+	    strcmp (dbus_message_get_path (message), DBUS_PATH_LOCAL) == 0) {
+		libhal_ctx_free (hal_ctx);
+		hal_ctx = NULL;
+		
+		dbus_connection_unref (dbus_connection);
+		dbus_connection = NULL;
+		
+		g_timeout_add (3000, reinit_dbus, NULL);
+		
+		return DBUS_HANDLER_RESULT_HANDLED;
+	}
+	
+	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
+static gboolean
+gpilotd_dbus_init (void)
+{
+	DBusError error;
+	
+	if (dbus_connection != NULL)
+		return TRUE;
+	
+	dbus_error_init (&error);
+	if (!(dbus_connection = dbus_bus_get (DBUS_BUS_SYSTEM, &error))) {
+		g_warning ("could not get system bus: %s", error.message);
+		dbus_error_free (&error);
+		return FALSE;
+	}
+	
+	dbus_connection_setup_with_g_main (dbus_connection, NULL);
+	dbus_connection_set_exit_on_disconnect (dbus_connection, FALSE);
+	
+	dbus_connection_add_filter (dbus_connection, gpilotd_dbus_filter_function, NULL, NULL);
+	
+	return TRUE;
+}
+
+
+static LibHalContext *
+gpilotd_hal_init (void)
+{
+	LibHalContext *ctx;
+	DBusError error;
+	char **devices;
+	int nr;
+	
+	if (!gpilotd_dbus_init ()) {
+		g_warning ("failed to initialise DBUS!");
+		return NULL;
+	}
+
+	if (!(ctx = libhal_ctx_new ())) {
+		g_warning ("failed to create a HAL context!");
+		return NULL;
+	}
+	
+	libhal_ctx_set_dbus_connection (ctx, dbus_connection);
+	
+	libhal_ctx_set_device_added (ctx, hal_device_added);
+	libhal_ctx_set_device_removed (ctx, hal_device_removed);
+	/*libhal_ctx_set_device_new_capability (ctx, hal_device_new_capability);*/
+	/*libhal_ctx_set_device_lost_capability (ctx, hal_device_lost_capability);*/
+	/*libhal_ctx_set_device_property_modified (ctx, hal_property_modified);*/
+	/*libhal_ctx_set_device_condition (ctx, hal_device_condition);*/
+	
+	dbus_error_init (&error);
+	if (!libhal_device_property_watch_all (ctx, &error)) {
+		g_warning ("failed to watch all HAL properties!: %s", error.message);
+		dbus_error_free (&error);
+		libhal_ctx_free (ctx);
+		return NULL;
+	}
+	
+	if (!libhal_ctx_init (ctx, &error)) {
+		g_warning ("libhal_ctx_init failed: %s", error.message);
+		dbus_error_free (&error);
+		libhal_ctx_free (ctx);
+		return NULL;
+	}
+	
+	/*
+	 * Do something to ping the HAL daemon - the above functions will
+	 * succeed even if hald is not running, so long as DBUS is.  But we
+	 * want to exit silently if hald is not running, to behave on
+	 * pre-2.6 systems.
+	 */
+	if (!(devices = libhal_get_all_devices (ctx, &nr, &error))) {
+		g_warning ("seems that HAL is not running: %s", error.message);
+		dbus_error_free (&error);
+		
+		libhal_ctx_shutdown (ctx, NULL);
+		libhal_ctx_free (ctx);
+		return NULL;
+	}
+	
+	libhal_free_string_array (devices);
+	
+	return ctx;
+}
+#endif /* WITH_HAL */
+#ifdef WITH_USB_VISOR
+#ifdef linux
+
 static gboolean 
 visor_devices_timeout (gpointer data) 
 {
 	GPilotContext *context = data;
 	GPilotDevice *device;
 	GList *l;
-	int i;
+	int i, use_sysfs;
+	static int devfs_warning = 0;
 	gboolean visor_exists = FALSE, visor_net = TRUE;
+	char *usbdevicesfile_str ="/proc/bus/usb/devices";
 	char line[256]; /* this is more than enough to fit any line from 
 			 * /proc/bus/usb/devices */
-
 	FILE *f;
+	gchar *fcontent;
+	gchar *fname;
+	gchar *vend_id; 
+	gchar *prod_id;
+	gchar *to_match;
+	GError *error = NULL;
+	regex_t regex_pattern;
+
+	GDir *topdir;
+	const gchar *entry;
+	gchar *sysfs_dir_name = "/sys/bus/usb/devices/";
+	gchar *f_vend = "idVendor";
+	gchar *f_prod = "idProduct";
+
 
 	g_assert (context != NULL);
 
@@ -880,29 +1263,102 @@ visor_devices_timeout (gpointer data)
 		return FALSE;
 
 	load_devices_xml ();
-	
-	/* Check /proc/bus/usb/devices for a usb device */
-	f = fopen ("/proc/bus/usb/devices", "r");
-	if (!f)
-		f = fopen ("/proc/bus/usb/devices_please-use-sysfs-instead", "r");
-	if (!f)
-		return TRUE;
-	
-	while (fgets (line, 255, f) != NULL && !visor_exists) {
-		if (line[0] != 'P')
-			continue;
-		
-		for (i = 0; i < vendor_product_ids->len; i++) {
-			if (!g_strncasecmp (line + strlen ("P:  "), 
-				      vendor_product_ids->pdata[i], 
-				      strlen (vendor_product_ids->pdata[i]))) {
+	/* choose a method for searching for known USB devices:
+	 * If we can't find sysfs, second choice is
+	 * legacy /proc/bus/usb/devices.
+	 */
+	use_sysfs = 1; /* default */
+	topdir = g_dir_open (sysfs_dir_name, 0, &error);
+	if (!topdir) {
+		use_sysfs = 0;
+		f = fopen (usbdevicesfile_str, "r");
+		if (!f)
+			f = fopen ("/proc/bus/usb/devices_please-use-sysfs-instead", "r");
+		if (!f) {
+			if (!devfs_warning) {
+				devfs_warning = 1;
+				char *str = g_strdup_printf (
+				    _("Failed to find directory %s or read file %s.  "
+					"Check that usbfs or sysfs is mounted."),
+				    sysfs_dir_name,
+				    usbdevicesfile_str);
+				g_warning (str);
+				g_free (str);
+			}
+			return TRUE; /* can't proceed */
+		}
+		devfs_warning = 0;
+	}
+	if (use_sysfs) {
+		/* This regex allows 99 root-hubs and a bus-depth of 6 tiers for
+		 * each root-hub. Each hub can have 99 ports. Refer to "Bus
+		 * Topology" in the USB 2.0 specs and the sysfs structures of
+		 * Linux USB for further explanation */
+		regcomp (&regex_pattern,
+		    "^[[:digit:]]{1,2}[-][[:digit:]]{1,2}([.][[:digit:]]{1,2}){0,5}$",
+		    REG_EXTENDED | REG_NOSUB); 
+
+		entry = g_dir_read_name (topdir);
+		while ((entry != NULL) && (!visor_exists)) {
+			if (!regexec (&regex_pattern, entry, 0, 0, 0)){
+				fname = g_build_filename (sysfs_dir_name, entry,
+				    f_vend, NULL);
+				if (!g_file_get_contents (fname, &fcontent,
+					NULL, &error)){
+					g_warning ("%s", &*error->message);
+					regfree (&regex_pattern);
+					g_free (fname);
+					g_dir_close (topdir);
+					return TRUE;
+				}
+				vend_id = g_strndup (fcontent, 4);
+				g_free (fname);
+
+				fname = g_build_filename (sysfs_dir_name, entry,
+				    f_prod, NULL);
+				if (!g_file_get_contents (fname, &fcontent,
+					NULL, &error)){
+					g_warning ("%s", &*error->message);
+					regfree (&regex_pattern);
+					g_free (fname);
+					g_free (vend_id);
+					g_dir_close (topdir);
+					return TRUE;
+				}
+				prod_id = g_strndup (fcontent, 4);
+				g_free (fname);
+
+				to_match = g_strconcat ("Vendor=", vend_id,
+				    " ProdID=", prod_id, NULL);
+				i = known_usb_device(to_match);
+				if(i != -1) {
+					visor_exists = TRUE;
+					visor_net = g_array_index (
+					    product_net, gboolean, i);
+				}
+				g_free (vend_id);
+				g_free (prod_id);
+				g_free (to_match);
+			}
+			entry = g_dir_read_name (topdir);
+		}
+		g_dir_close (topdir);
+		regfree (&regex_pattern);
+	} else {
+		/* non sysfs branch... read /proc/bus/usb/devices */
+		while (fgets (line, 255, f) != NULL && !visor_exists) {
+			if (line[0] != 'P')
+				continue;
+			i = known_usb_device(line + 4); /* line + strlen("P:  ") */
+			if (i != -1) {
 				visor_exists = TRUE;
-				visor_net = g_array_index (product_net, gboolean, i);
+				visor_net = g_array_index (
+				    product_net, gboolean, i);
 			}
 		}
-	}
 	
-	fclose (f);
+		fclose (f);
+	}
 	
 	if (visor_exists) {
 		l = context->devices;
@@ -916,9 +1372,11 @@ visor_devices_timeout (gpointer data)
 				 * the kernel guys this is the best way to 
                                  * go. */
 				sync_device (device, context);
+				sleep(1);
 
 				if (!visor_net)
 					device->type = PILOT_DEVICE_USB_VISOR;
+				break; /* don't try to sync any more devices! */
 			}
 			l = l->next;
 		}
@@ -927,10 +1385,12 @@ visor_devices_timeout (gpointer data)
 	return TRUE;
 }
 
-#endif
-#endif
+#endif /* WITH_USB_VISOR */
+#endif /* linux */
 
-void monitor_channel (GPilotDevice *dev,GPilotContext *context) 
+
+static void
+monitor_channel (GPilotDevice *dev, GPilotContext *context) 
 {
 	g_assert (context != NULL);
 	
@@ -958,37 +1418,42 @@ void monitor_channel (GPilotDevice *dev,GPilotContext *context)
 		g_assert_not_reached ();
 #endif /* WITH_NETWORK */
 	} if (dev->type == PILOT_DEVICE_USB_VISOR) {
+		if(hal_initialised) {
+			/* handled by hal callbacks */
+			dev->device_exists = FALSE;
+		} else {
 #ifdef WITH_USB_VISOR
 #ifdef linux
-		/* We want to watch the /proc/bus/usb/devices file once 
-		 * per context, and then check all devices each time it is
-		 * woken up. */
-		if (visor_timeout_id == -1) {
-			visor_timeout_id = g_timeout_add (2000, visor_devices_timeout, context);
-		}
-#else /* linux*/
-		g_assert_not_reached ();
+			/* We want to watch for a new recognised USB device
+			 * once per context. */
+			if (visor_timeout_id == -1) {
+				visor_timeout_id = g_timeout_add (2000,
+				    visor_devices_timeout, context);
+			}
+#else /* linux */
+			g_assert_not_reached ();
 #endif /* linux */
 #endif /* WITH_USB_VISOR */
-		dev->device_exists = FALSE;
+		}
 	}
 
 	if (dev->type == PILOT_DEVICE_NETWORK) {
-		g_message (_("Watching %s (%s, %s)"), dev->name, dev->ip, dev->host);
+		g_message (_("Watching %s (%s)"), dev->name, dev->ip);
 	} else {
-		g_message (_("Watching %s (%s)"),dev->name,dev->port);
+		g_message (_("Watching %s (%s)"), dev->name, dev->port);
 	}
 }
 
 static void 
 sig_hup_handler (int dummy)
 {
-	signal (SIGHUP,sig_hup_handler);
-	reread_config=TRUE;
+	signal (SIGHUP, sig_hup_handler);
+	reread_config = TRUE;
 }
 
 static void 
-sig_term_handler (int dummy) {
+sig_term_handler (int dummy)
+{
 	g_message (_("Exiting (caught SIGTERM)..."));
 	remove_pid_file ();
 	gpilotd_corba_quit ();
@@ -996,7 +1461,8 @@ sig_term_handler (int dummy) {
 }
 
 static void 
-sig_int_handler (int dummy) {
+sig_int_handler (int dummy)
+{
 	g_message (_("Exiting (caught SIGINT)..."));
 	remove_pid_file ();
 	gpilotd_corba_quit ();
@@ -1004,23 +1470,14 @@ sig_int_handler (int dummy) {
 }
 
 /* This deletes the ~/.gpilotd.pid file */
-void 
-remove_pid_file () {
-	const gchar *home_directory;
-	gchar *pid_file;
-
-	home_directory=g_get_home_dir ();
-	if (home_directory) {
-		pid_file=(gchar *)g_malloc (strlen (home_directory) + 
-					   strlen ("/.gpilotd.pid") + 1);
-		strcpy (pid_file,home_directory);
-		strcat (pid_file,"/.gpilotd.pid");
-
-		if (access (pid_file,R_OK|W_OK)==0) {
-			unlink (pid_file);
-		} 
-		g_free (pid_file);
-	}
+static void 
+remove_pid_file (void)
+{
+	char *filename;
+	
+	filename = g_build_filename (g_get_home_dir (), ".gpilotd.pid", NULL);
+	unlink (filename);
+	g_free (filename);
 }
 
 /*
@@ -1029,43 +1486,43 @@ remove_pid_file () {
    SIGHUPS
 */
 static void 
-write_pid_file ()
+write_pid_file (void)
 {
-	gchar *queue_dir;
-	queue_dir = g_strdup_printf ("%s/.gpilotd", g_get_home_dir ());
-
-	if (g_get_home_dir ()) {
-		gchar *pid_file;
-		int fd;
-
-		pid_file=(gchar *)g_malloc (strlen (g_get_home_dir ()) +
-					    strlen ("/.gpilotd.pid") + 1);
-		strcpy (pid_file, g_get_home_dir());
-		strcat (pid_file,"/.gpilotd.pid");
-		if ((fd=open (pid_file,O_RDWR | O_TRUNC | O_CREAT, 0644)) >= 0) {
-			gchar pid[50]; /* If the pid is > 50 digits we have problems :) */
-			if (sizeof (pid_t) == sizeof (int)) {
-				g_snprintf (pid,50,"%d\n",getpid ());
-			} else { /* Assume pid_t is sizeof long */
-				g_snprintf (pid,50,"%ld\n",(unsigned long)getpid ());
-			}
-			write (fd,pid,strlen (pid));
-			close (fd);
-		} else {
-			g_warning (_("Unable to open or create "
-				    "file %s with read/write privs"),pid_file);
-		}
-
-		g_free (pid_file);
-	} else {
-		g_warning (_("Unable to find home directory for uid %d"),geteuid ());
-	}
-	if (g_file_test (queue_dir, G_FILE_TEST_IS_DIR)==FALSE) {
-		if (mkdir (queue_dir, 0700)) {
-			g_warning (_("Unable to create file installation queue directory"));
-		}
-	}
-	g_free (queue_dir);
+	char *filename, *dirname, *buf;
+	size_t nwritten = 0;
+	ssize_t n, w;
+	int fd;
+	
+	dirname = g_build_filename (g_get_home_dir (), ".gpilotd", NULL);
+	if (!g_file_test (dirname, G_FILE_TEST_IS_DIR) && mkdir (dirname, 0777) != 0)
+		g_warning (_("Unable to create file installation queue directory"));
+	g_free (dirname);
+	
+	filename = g_build_filename (g_get_home_dir (), ".gpilotd.pid", NULL);
+	fd = open (filename, O_CREAT | O_TRUNC | O_RDWR, 0644);
+	g_free (filename);
+	
+	if (fd == -1)
+		return;
+	
+	buf = g_strdup_printf ("%lu", (unsigned long) getpid ());
+	n = strlen (buf);
+	
+	do {
+		do {
+			w = write (fd, buf + nwritten, n - nwritten);
+		} while (w == -1 && errno == EINTR);
+		
+		if (w == -1)
+			break;
+		
+		nwritten += w;
+	} while (nwritten < n);
+	
+	/* FIXME: what do we do if the write is incomplete? (e.g. nwritten < n)*/
+	
+	fsync (fd);
+	close (fd);
 }
 
 /*  
@@ -1078,13 +1535,14 @@ write_pid_file ()
 */
 
 static void 
-wait_for_sync_and_sync (GPilotContext *context) {
-	signal (SIGTERM,sig_term_handler);
-	signal (SIGINT,sig_int_handler);
-	signal (SIGHUP,sig_hup_handler);
-
+wait_for_sync_and_sync (void)
+{
+	signal (SIGTERM, sig_term_handler);
+	signal (SIGINT, sig_int_handler);
+	signal (SIGHUP, sig_hup_handler);
+	
 	g_list_foreach (context->devices, (GFunc)monitor_channel, context);
-
+	
 	while (1) {
 		if (reread_config) {
 			g_message (_("Shutting down devices"));
@@ -1102,7 +1560,7 @@ wait_for_sync_and_sync (GPilotContext *context) {
 /* This function display which pilot-link version was used
    and which features were enabled at compiletime */
 static void
-dump_build_info ()
+dump_build_info (void)
 {
 	GString *str = g_string_new (NULL);
 	g_message ("compiled for pilot-link version %s",
@@ -1128,45 +1586,39 @@ dump_build_info ()
 int 
 main (int argc, char *argv[])
 {
-	GPilotContext *context;
-
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-
+	
 	/*g_log_set_always_fatal (G_LOG_LEVEL_ERROR |
 				G_LOG_LEVEL_CRITICAL |
 				G_LOG_LEVEL_WARNING);*/
-
+	
 	/* Intro */
 	g_message ("%s %s starting...",PACKAGE,VERSION);
 	dump_build_info ();
-
+	
 	gnome_init (PACKAGE, VERSION, argc, argv);
+
+#ifdef WITH_HAL
+	if ((hal_ctx = gpilotd_hal_init ()) != NULL)
+		hal_initialised = 1; /* if 0, fall back to polling sysfs */
+#endif
 	
 	/* Setup the correct gpilotd.pid file */
 	remove_pid_file ();
 	write_pid_file ();
-
+	
 	/* Init corba and context, this call also loads the config into context */
 	g_type_init ();
-	gpilotd_corba_init (&argc,argv,&context);
-
-	reread_config=FALSE;	
-
+	gpilotd_corba_init (&argc, argv, &context);
+	
 	/* Begin... */
-	wait_for_sync_and_sync (context);
-
+	wait_for_sync_and_sync ();
+	
 	/* It is unlikely that we will end here */
 	remove_pid_file ();
 	gpilotd_corba_quit ();
 	
 	return 0;
 }
-
-
-
-
-
-
-
